#include <time.h>
#include <socket.h>
#include <in.h>
#include <network.h>

/* Select routine.  */
int
VMSselect (nfds, readfds, writefds, exceptfds, timeout)
     int nfds;
     fd_set *readfds, *writefds, *exceptfds;
     struct timeval *timeout;
{
  int timer, fd, alarm_set, total, end;
  long mask, cluster;
  struct descriptor termdesc;
  static fd_set new_readfds, new_writefds, new_exceptfds;

  FD_ZERO (&new_readfds);
  FD_ZERO (&new_writefds);
  FD_ZERO (&new_exceptfds);
  total = 0;

  /* Assign a terminal channel if we haven't already.  */
  if (terminal.chan == -1)
    {
      termdesc.size = 10;
      termdesc.ptr = "SYS$INPUT:";
      sys$assign (&termdesc, &terminal.chan, 0, 0);
    }
  alarm_set = 0;
  if (timeout != NULL)
    {
      /* If a timeout is given then set the alarm.  */
      end = timeout->tv_sec;
      if (timer != 0)
	{
	  /* We need to reset the alarm if it didn't fire, but we set it.  */
	  alarm_set = 1;
	  si_alarm (end);
	}
    }
  else
    end = 1;

  do
    {
      if (exceptfds)
	{
	   /* Nothing */ ;
	}

      if (writefds)
	{
	  for (fd = 0; fd < nfds; fd++)
	    if (FD_ISSET (fd, writefds))
	      {
		if (p[fd].connect_pending)
		   /* Nothing */ ;
		else if ((p[fd].status == ACTIVE_CONNECTION)
			 || (p[fd].status == PASSIVE_CONNECTION))
		  {
		    FD_SET (fd, &new_writefds);
		    total++;
		  }
	      }
	}

      if (readfds)
	{
	  /* True if data pending or an accept.  */
	  for (fd = 3; fd < nfds; fd++)
	    if (FD_ISSET (fd, readfds) &&
		((p[fd].fd_buff_size != -1) || (p[fd].accept_pending == 1)))
	      {
		FD_SET (fd, &new_readfds);
		total++;
	      }
	}

      if (total || (end == 0))
	break;

      /* Otherwise, wait on an event flag.  It's possible that the wait can
	 be stopped by a spurious event flag being set -- i.e. one that's
	 got a status not normal.  So we've got to be prepared to loop
	 around the wait until a valid reason happens.  */

      /* Set up the wait mask.  */
      cluster = 0;
      mask = 0;
      for (fd = 3; fd < nfds; fd++)
	{
	  sys$clref (fd);
	  if (readfds)
	    if FD_ISSET
	      (fd, readfds) mask |= (1 << fd);
	  if (writefds)
	    if FD_ISSET
	      (fd, writefds) mask |= (1 << fd);
	  if (exceptfds)
	    if FD_ISSET
	      (fd, exceptfds) mask |= (1 << fd);
	}

      mask |= (1 << TIMER_EFN);

      /* Clear it off just in case.  */
      sys$clref (TIMER_EFN);

      /* Wait around.  */
      sys$wflor (cluster, mask);

      mask = 0;
      if (read_efn (TIMER_EFN))
	{
	  errno = EINTR;
	  break;
	}
  } while (1);
  /*NOTREACHED*/

  /* Unset the alarm if we set it.  */
  if (alarm_set == 1)
    alarm (0);

  if (readfds)
    *readfds = new_readfds;

  if (writefds)
    *writefds = new_writefds;

  if (exceptfds)
    *exceptfds = new_exceptfds;

  return total;
}

